library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity axi_writer_16 is
  generic (
    BUF_WIDTH : integer := 16
  );
  port (
    clk : in std_logic;
    reset : in std_logic;
    mem_din : in std_logic_vector(16-1 downto 0);
    mem_we : out std_logic;
    mem_address : out std_logic_vector(32-1 downto 0);
    mem_wclk : out std_logic;
    S_AXI_AWADDR : out std_logic_vector(32-1 downto 0);
    S_AXI_AWLEN : out std_logic_vector(8-1 downto 0);
    S_AXI_AWVALID : out std_logic;
    S_AXI_AWSIZE : out std_logic_vector(3-1 downto 0);
    S_AXI_AWBURST : out std_logic_vector(2-1 downto 0);
    S_AXI_AWCACHE : out std_logic_vector(4-1 downto 0);
    S_AXI_AWPROT : out std_logic_vector(3-1 downto 0);
    S_AXI_AWREADY : in std_logic;
    S_AXI_WDATA : out std_logic_vector(16-1 downto 0);
    S_AXI_WLAST : out std_logic;
    S_AXI_WVALID : out std_logic;
    S_AXI_WREADY : in std_logic;
    S_AXI_WSTRB : out std_logic_vector(2-1 downto 0);
    S_AXI_BRESP : in std_logic_vector(2-1 downto 0);
    S_AXI_BVALID : in std_logic;
    S_AXI_BREADY : out std_logic;
    request : in std_logic;
    busy : out std_logic;
    addr : in std_logic_vector(32-1 downto 0);
    len : in std_logic_vector(32-1 downto 0);
    debug : out std_logic_vector(8-1 downto 0)
  );
end axi_writer_16;

architecture RTL of axi_writer_16 is

  attribute mark_debug : string;
  attribute keep : string;
  attribute S : string;


  signal clk_sig : std_logic;
  signal reset_sig : std_logic;
  signal mem_din_sig : std_logic_vector(16-1 downto 0);
  signal mem_we_sig : std_logic := '0';
  signal mem_address_sig : std_logic_vector(32-1 downto 0) := (others => '0');
  signal mem_wclk_sig : std_logic;
  signal S_AXI_AWADDR_sig : std_logic_vector(32-1 downto 0) := (others => '0');
  signal S_AXI_AWLEN_sig : std_logic_vector(8-1 downto 0) := (others => '0');
  signal S_AXI_AWVALID_sig : std_logic := '0';
  signal S_AXI_AWSIZE_sig : std_logic_vector(3-1 downto 0);
  signal S_AXI_AWBURST_sig : std_logic_vector(2-1 downto 0);
  signal S_AXI_AWCACHE_sig : std_logic_vector(4-1 downto 0);
  signal S_AXI_AWPROT_sig : std_logic_vector(3-1 downto 0);
  signal S_AXI_AWREADY_sig : std_logic;
  signal S_AXI_WDATA_sig : std_logic_vector(16-1 downto 0) := (others => '0');
  signal S_AXI_WLAST_sig : std_logic := '0';
  signal S_AXI_WVALID_sig : std_logic := '0';
  signal S_AXI_WREADY_sig : std_logic;
  signal S_AXI_WSTRB_sig : std_logic_vector(2-1 downto 0);
  signal S_AXI_BRESP_sig : std_logic_vector(2-1 downto 0);
  signal S_AXI_BVALID_sig : std_logic;
  signal S_AXI_BREADY_sig : std_logic := '0';
  signal request_sig : std_logic;
  signal busy_sig : std_logic := '0';
  signal addr_sig : std_logic_vector(32-1 downto 0);
  signal len_sig : std_logic_vector(32-1 downto 0);
  signal debug_sig : std_logic_vector(8-1 downto 0) := (others => '0');

  signal write_counter : signed(32-1 downto 0) := (others => '0');
  signal next_read_addr : signed(32-1 downto 0) := (others => '0');
  signal tmp_0001 : std_logic;
  signal tmp_0002 : std_logic;
  signal tmp_0003 : std_logic;
  type Type_main is (
    main_IDLE,
    main_init,
    main_write0,
    main_write_pre,
    main_write,
    main_write_next  
  );
  signal main : Type_main := main_IDLE;
  signal main_delay : signed(32-1 downto 0) := (others => '0');
  signal tmp_0004 : signed(32-1 downto 0);
  signal tmp_0005 : signed(8-1 downto 0);
  signal tmp_0006 : std_logic;
  signal tmp_0007 : signed(32-1 downto 0);
  signal tmp_0008 : signed(32-1 downto 0);
  signal tmp_0009 : std_logic;
  signal tmp_0010 : std_logic;
  signal tmp_0011 : signed(32-1 downto 0);
  signal tmp_0012 : std_logic;
  signal tmp_0013 : std_logic;
  signal tmp_0014 : std_logic;
  signal tmp_0015 : std_logic;
  signal tmp_0016 : std_logic;
  signal tmp_0017 : std_logic;

begin

  clk_sig <= clk;
  reset_sig <= reset;
  mem_din_sig <= mem_din;
  mem_we <= mem_we_sig;
  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        mem_we_sig <= '0';
      else
        if main = main_write0 then
          mem_we_sig <= '1';
        elsif main = main_write_pre then
          mem_we_sig <= '0';
        elsif main = main_write then
          mem_we_sig <= tmp_0010;
        elsif main = main_write_next then
          mem_we_sig <= '0';
        end if;
      end if;
    end if;
  end process;

  mem_address <= mem_address_sig;
  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        mem_address_sig <= (others => '0');
      else
        if main = main_write0 then
          mem_address_sig <= (others => '0');
        elsif main = main_write then
          mem_address_sig <= std_logic_vector(next_read_addr);
        end if;
      end if;
    end if;
  end process;

  mem_wclk <= mem_wclk_sig;
  mem_wclk_sig <= clk_sig;

  S_AXI_AWADDR <= S_AXI_AWADDR_sig;
  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        S_AXI_AWADDR_sig <= (others => '0');
      else
        if main = main_init then
          S_AXI_AWADDR_sig <= addr_sig;
        end if;
      end if;
    end if;
  end process;

  S_AXI_AWLEN <= S_AXI_AWLEN_sig;
  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        S_AXI_AWLEN_sig <= (others => '0');
      else
        if main = main_init then
          S_AXI_AWLEN_sig <= std_logic_vector(tmp_0005);
        end if;
      end if;
    end if;
  end process;

  S_AXI_AWVALID <= S_AXI_AWVALID_sig;
  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        S_AXI_AWVALID_sig <= '0';
      else
        if main = main_init then
          S_AXI_AWVALID_sig <= '1';
        elsif main = main_write0 then
          S_AXI_AWVALID_sig <= tmp_0006;
        end if;
      end if;
    end if;
  end process;

  S_AXI_AWSIZE <= S_AXI_AWSIZE_sig;
  S_AXI_AWSIZE_sig <= "001";

  S_AXI_AWBURST <= S_AXI_AWBURST_sig;
  S_AXI_AWBURST_sig <= "01";

  S_AXI_AWCACHE <= S_AXI_AWCACHE_sig;
  S_AXI_AWCACHE_sig <= X"3";

  S_AXI_AWPROT <= S_AXI_AWPROT_sig;
  S_AXI_AWPROT_sig <= "000";

  S_AXI_AWREADY_sig <= S_AXI_AWREADY;
  S_AXI_WDATA <= S_AXI_WDATA_sig;
  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        S_AXI_WDATA_sig <= (others => '0');
      else
        if main = main_write then
          S_AXI_WDATA_sig <= mem_din_sig;
        end if;
      end if;
    end if;
  end process;

  S_AXI_WLAST <= S_AXI_WLAST_sig;
  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        S_AXI_WLAST_sig <= '0';
      else
        if main = main_write then
          S_AXI_WLAST_sig <= tmp_0009;
        elsif main = main_write_next then
          S_AXI_WLAST_sig <= tmp_0012;
        end if;
      end if;
    end if;
  end process;

  S_AXI_WVALID <= S_AXI_WVALID_sig;
  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        S_AXI_WVALID_sig <= '0';
      else
        if main = main_write then
          S_AXI_WVALID_sig <= '1';
        elsif main = main_write_next then
          S_AXI_WVALID_sig <= tmp_0013;
        end if;
      end if;
    end if;
  end process;

  S_AXI_WREADY_sig <= S_AXI_WREADY;
  S_AXI_WSTRB <= S_AXI_WSTRB_sig;
  S_AXI_WSTRB_sig <= "11";

  S_AXI_BRESP_sig <= S_AXI_BRESP;
  S_AXI_BVALID_sig <= S_AXI_BVALID;
  S_AXI_BREADY <= S_AXI_BREADY_sig;
  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        S_AXI_BREADY_sig <= '0';
      else
        if main = main_IDLE then
          S_AXI_BREADY_sig <= '1';
        end if;
      end if;
    end if;
  end process;

  request_sig <= request;
  busy <= busy_sig;
  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        busy_sig <= '0';
      else
        if main = main_IDLE then
          busy_sig <= tmp_0014;
        end if;
      end if;
    end if;
  end process;

  addr_sig <= addr;
  len_sig <= len;
  debug <= debug_sig;
  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        debug_sig <= (others => '0');
      else
        if main = main_IDLE then
          debug_sig <= X"00";
        elsif main = main_init then
          debug_sig <= X"01";
        elsif main = main_write0 then
          debug_sig <= X"02";
        elsif main = main_write_pre then
          debug_sig <= X"03";
        elsif main = main_write then
          debug_sig <= X"04";
        elsif main = main_write_next then
          debug_sig <= X"05";
        end if;
      end if;
    end if;
  end process;


  -- expressions
  tmp_0001 <= '1' when S_AXI_WREADY_sig = '1' else '0';
  tmp_0002 <= '1' when write_counter = 0 else '0';
  tmp_0003 <= '1' when S_AXI_AWREADY_sig = '1' else '0';
  tmp_0004 <= signed(len_sig) - 1;
  tmp_0005 <= tmp_0004(32 - 24 - 1 downto 0);
  tmp_0006 <= '0' when tmp_0003 = '1' else '1';
  tmp_0007 <= next_read_addr + 1;
  tmp_0008 <= write_counter - 1;
  tmp_0009 <= '1' when write_counter = 1 else '0';
  tmp_0010 <= '0' when tmp_0009 = '1' else '1';
  tmp_0011 <= next_read_addr + 1;
  tmp_0012 <= '0' when tmp_0001 = '1' else S_AXI_WLAST_sig;
  tmp_0013 <= '0' when tmp_0001 = '1' else S_AXI_WVALID_sig;
  tmp_0014 <= '1' when request_sig = '1' else '0';
  tmp_0015 <= tmp_0001 and tmp_0002;
  tmp_0016 <= not tmp_0002;
  tmp_0017 <= tmp_0001 and tmp_0016;

  -- sequencers
  process (clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        main <= main_IDLE;
        main_delay <= (others => '0');
      else
        case (main) is
          when main_IDLE => 
            if tmp_0014 = '1' then
              main <= main_init;
            end if;
          when main_init => 
            main <= main_write0;
          when main_write0 => 
            if tmp_0003 = '1' then
              main <= main_write_pre;
            end if;
          when main_write_pre => 
            main <= main_write;
          when main_write => 
            main <= main_write_next;
          when main_write_next => 
            if tmp_0015 = '1' then
              main <= main_IDLE;
            elsif tmp_0017 = '1' then
              main <= main_write;
            end if;
          when others => null;
        end case;
      end if;
    end if;
  end process;


  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        write_counter <= (others => '0');
      else
        if main = main_init then
          write_counter <= signed(len_sig);
        elsif main = main_write then
          write_counter <= tmp_0008;
        end if;
      end if;
    end if;
  end process;

  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        next_read_addr <= (others => '0');
      else
        if main = main_IDLE then
          next_read_addr <= 0;
        elsif main = main_write0 then
          next_read_addr <= tmp_0007;
        elsif main = main_write then
          next_read_addr <= tmp_0011;
        end if;
      end if;
    end if;
  end process;



end RTL;
